/*********************************************************************************
**
**   $Id: InvoiceWindow.ui.h,v 1.25 2004/07/17 14:24:11 croftj Exp $
**   Copyright (c) 2004 Joe Croft <joe@croftj.net>
**   
**   This file is part of ah2.
**
**   ah2 is free software; you can redistribute it and/or modify
**   it under the terms of the GNU General Public License as published by
**   the Free Software Foundation; either version 2 of the License, or
**   (at your option) any later version.
**
**   Foobar is distributed in the hope that it will be useful,
**   but WITHOUT ANY WARRANTY; without even the implied warranty of
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**   GNU General Public License for more details.
**
**   You should have received a copy of the GNU General Public License
**   along with Foobar; if not, write to the Free Software
**   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
*********************************************************************************/

// #include <Data/DataForm.h>
#include <qapplication.h>
#include <qfont.h>
#include <qfontdialog.h>
#include <qmessagebox.h>
#include <qprocess.h>
#include <qsettings.h>
#include <qsqldatabase.h>
#include <qsqlrecord.h>
#include <qaccel.h>

#include "bizinfo.h"
#include "configuration.h"
#include "confdefs.h"
#include "customerentry.h"
#include "custvehcselect.h"
#include "custvehcedit.h"
#include "wopartsentry.h"
#include "previewworkorder.h"
#include "laborentry.h"
#include "movetransactions.h"
#include "partsedit.h"
#include "reportgen.h"
#include "vendorentry.h"
#include "workorderselect.h"
#include "WorkOrderTable.h"
#include "WOLaborForm.h"
#include "WOLaborTable.h"
#include "WOPartsForm.h"
#include "WOPartsTable.h"

extern QApplication *a;
extern QSettings Config;
extern QStringList Reports;
extern QSqlDatabase *autohouse;

void InvoiceWindow::openWorkOrder()
{
   printf("InvoiceWindow::openWorkOrder(): Enter\n");
   WorkOrderSelect *wo = new WorkOrderSelect();
//   connect((QObject*)(wo->WorkOrders), SIGNAL(currentChanged(QSqlRecord *)), this, SLOT(refresh(QSqlRecord *)));
   connect(wo, SIGNAL(currentChanged(QSqlRecord *)), this, SLOT(refresh(QSqlRecord *)));
   QAccel *wo_canceled = new QAccel(wo);
   wo_canceled->connectItem(wo_canceled->insertItem(Qt::Key_Escape), wo, SLOT(workorderRejected()));
   QAccel *wo_selected = new QAccel((QWidget*)(wo->WorkOrders));
   wo_selected->connectItem(wo_selected->insertItem(Qt::Key_Return), wo, SLOT(workorderSelected()));
   wo_selected->connectItem(wo_selected->insertItem(Qt::Key_Enter), wo, SLOT(workorderSelected()));
   wo->refresh();
   wo->sortBy("date_in DESC");
   wo->sortBy("name ASC");
   wo->WorkOrders->selectRow(0);
   wo->showMaximized();
   wo->exec();
   delete wo;
   printf("InvoiceWindow::openWorkOrder(): Exit\n");
}

void InvoiceWindow::refresh(QSqlRecord *r)
{
   printf("InvoiceWindow::refresh(QSqlRecord): Enter\n");
   refresh(r->value("ident").toString());
   printf("InvoiceWindow::refresh(QSqlRecord): Exit\n");
}

void InvoiceWindow::refresh(QString wo)
{
   double partsTotal, labor, taxRate, tax, hazmat;
   QString strPartsTotal, strTax;
   bool taxLabor, taxParts, taxHazMat;

   printf("InvoiceWindow::refresh(): Enter\n");
   QString sql = "select ident, customer, vehicle, parts, labor, r_tax, hazmat from workorders where ident = '" + wo + "'";
   printf("InvoiceWindow::refresh(): sql = |%s|\n", (const char*)sql);
   QSqlQuery wo_q(sql);
   if ( !wo_q.isActive() || !wo_q.next()) 
   {
      QMessageBox::warning(NULL, "Error!", "Could not find workorder '" + wo + "': \n" + wo_q.lastError().driverText() + wo_q.lastError().databaseText(), "Okay");
      return;
   }
   QString cust = wo_q.value(1).toString();
   QString vehicle = wo_q.value(2).toString();
   PartsTotal->setText(MoneyEdit::formatCurrency(wo_q.value(3).toDouble()));
   LaborTotal->setText(MoneyEdit::formatCurrency(wo_q.value(4).toDouble()));

   labor = wo_q.value(4).toDouble();
   taxRate = wo_q.value(5).toDouble();
   hazmat = wo_q.value(6).toDouble();

   sql = "select sum( cast(qty as double precision) * cast(cost as double precision) ) from wo_parts where taxable = 'Y' and workorder = '" + wo + "'";
   printf("InvoiceWindow::refresh(): sql = |%s|\n", (const char*)sql);
   QSqlQuery wo_q1(sql);
   if ( wo_q1.isActive() && wo_q1.next()) 
      partsTotal = wo_q1.value(0).toDouble();
   else 
      partsTotal = 0.0;

   
   taxParts = Config.readEntry(CONFIG_ENTRY_TAX_PARTS, "Y") == "Y";
   taxLabor = Config.readEntry(CONFIG_ENTRY_TAX_LABOR, "N") == "Y";
   taxHazMat = Config.readEntry(CONFIG_ENTRY_TAX_HAZMAT, "N") == "Y";

   tax =  (taxParts == true) ? taxRate * partsTotal : 0.00;
   tax += (taxLabor == true) ? taxRate * labor : 0.00;
   tax += (taxHazMat == true) ? taxRate * hazmat : 0.00;

   strTax.setNum(tax);
   sql = "update workorders set tax = '" + strTax + "' where ident = '" + wo + "'";
   printf("InvoiceWindow::refresh(): sql = |%s|\n", (const char*)sql);
   QSqlQuery wo_q2(sql);
   wo_q2.exec();

   sql = "update workorders set total = parts + labor + tax + hazmat where ident = '" + wo + "'";
   printf("InvoiceWindow::refresh(): sql = |%s|\n", (const char*)sql);
   wo_q2.prepare(sql);
   wo_q2.exec();

   Customer->refresh(cust);
   Vehicle->refresh(vehicle);
   Customer->setReadOnly(true);
   Vehicle->setReadOnly(true);
   WorkOrderEntry->refresh(wo);
   paintPartsWindow(wo);
   paintLaborWindow(wo);
   haveWorkOrder = true;
   printf("InvoiceWindow::refresh(): Exit\n");
}

void InvoiceWindow::updateRecord()
{
   printf("InvoiceWindow::updateRecord(): Enter\n");

   printf("InvoiceWindow::updateRecord(): Workorder closed? %d\n", WorkOrderEntry->isClosed());
   if ( WorkOrderEntry->isClosed() ) 
   {
      QString s = "<qt><p><nobr>This workorder has already been closed!</nobr><br><nobr><b>Updating it will affect your accounting.</b></nobr></p><p><nobr>Are you sure you want to update it?</nobr></p></qt>";
      if (QMessageBox::critical(NULL, "Updating Closed Work Order", s, QMessageBox::Yes, 
                                 QMessageBox::No | QMessageBox::Escape | QMessageBox::Default) == QMessageBox::No) 
      {
         printf("InvoiceWindow::updateRecord(): Exit\n");
         return;
      }
   }
   Customer->updateRecord();
   Customer->commitTransaction();
   Vehicle->updateRecord();
   Vehicle->commitTransaction();
   WorkOrderEntry->updateRecord();
   WorkOrderEntry->commitTransaction();

   QString sql = "select sum(cost) from wo_labor_v where workorder = '" + WorkOrderEntry->getWorkorder() + "'";
   QSqlQuery q(sql);
   if ( q.isActive() ) 
   {
      while (q.next()) 
      {
         sql = "update workorders set labor = '" + q.value(0).toString() + "' where ident = '" + WorkOrderEntry->getWorkorder() + "'"; 
         QSqlQuery q1(sql);
         q1.exec();
      }
   }

   refresh(WorkOrderEntry->getWorkorder());
   printf("InvoiceWindow::updateRecord(): Exit\n");
}

void InvoiceWindow::paintPartsWindow(QString workorder)
{
   QString text = "";
   printf("InvoiceWindow::paintPartsWindow(): Enter\n");
   QString sql = "select pn, descr, qty, cost, cast(cast(qty as numeric) * cost as numeric(15,2)) from wo_parts where workorder = '" + 
                  workorder + "' order by ident";
   printf("InvoiceWindow::paintPartsWindow(): sql = |%s|\n", (const char*)sql);
   QSqlQuery q(sql);
   if ( q.isActive() ) 
   {
      while (q.next()) 
      {
         if ( text == "" ) 
            text = "<table width=\"100%\"><tr><th><b>Part #</b></th><th><b>Description</b></th><th align=\"right\"><b>Qty</b></th><th align=\"right\"><b>Cost</b></th><th align=\"right\"><b>Total</b></th></tr>\n";
         text += "<tr>\n";
         text += "<th>" + q.value(0).toString() + "</th>\n";
         text += "<th>" + q.value(1).toString() + "</th>\n";
         text += "<th align=\"right\">" + q.value(2).toString() + "</th>\n";
         text += "<th align=\"right\">" + MoneyEdit::formatCurrency(q.value(3).toDouble()) + "</th>\n";
         text += "<th align=\"right\"><b>" + MoneyEdit::formatCurrency(q.value(4).toDouble()) + "</b></th>\n";
         text += "</tr>\n";
      }
   }
   if ( text != "" ) 
      text += "</table>\n";
   PartsWindow->setText(text);
   printf("InvoiceWindow::paintPartsWindow(): Exit\n");
}

void InvoiceWindow::paintLaborWindow(QString workorder)
{
   QString text = "";
   printf("InvoiceWindow::paintLaborWindow(): Enter\n");
   QString sql = "select wo_labor.hours, wo_labor.descr, cast(cast(wo_labor.hours as numeric) * workorders.r_labor as numeric(15,2)) from wo_labor, workorders where workorder='" +
                  workorder + "' and workorders.ident='" + workorder + "' order by wo_labor.ident";
   printf("InvoiceWindow::paintLaborWindow(): sql = |%s|\n", (const char*)sql);
   QSqlQuery q(sql);
   if ( q.isActive() ) 
   {
      while (q.next()) 
      {
         if ( text == "" ) 
            text = "<table width=\"100%\"><tr><th align=\"right\"><b>Hours</b></th><th><b>Description</b></th><th align=\"right\"><b>Total</b></th></tr>\n";
         text += "<tr>\n";
         text += "<th align=\"right\">" + q.value(0).toString() + "</th>\n";
         text += "<th>" + q.value(1).toString() + "</th>\n";
         text += "<th align=\"right\"><b>" + MoneyEdit::formatCurrency(q.value(2).toDouble()) + "</b></th>\n";
         text += "</tr>\n";
      }
   }
   if ( text != "" ) 
      text += "</table>\n";
   LaborWindow->setText(text);
   printf("InvoiceWindow::paintLaborWindow(): Exit\n");
}

void InvoiceWindow::workorderSelected()
{

}

void InvoiceWindow::workorderRejected()
{

}

void InvoiceWindow::editCustomers()
{
   CustomerEntry *c = new CustomerEntry();
   c->initialize();
   c->exec();
   delete c;
}

void InvoiceWindow::editVehicles()
{
   CustVehcEdit *c = new CustVehcEdit();
   c->initialize();
   c->exec();
   delete c;
}

void InvoiceWindow::createWorkOrder()
{
   QString owner, vehicle, labor_rate, tax_rate;

   printf("InvoiceWindow::createWorkOrder(): Enter\n");
   fflush(stdout);
   CustVehcSelect *cvs = new CustVehcSelect(this);
   cvs->showMaximized();
   vehicle.setNum(cvs->exec());
   delete cvs;
   if ( vehicle.toInt() <= 0 ) 
      return;
   QString sql = "select owner from vehicles where ident = '" + vehicle + "'";
   printf("InvoiceWindow::createWorkOrder(): sql = |%s|\n", (const char*)sql);
   QSqlQuery vehc_q(sql);
   if ( vehc_q.isActive() && vehc_q.next()) 
   {
      owner = vehc_q.value(0).toString();
      sql = QString("insert into workorders (customer, vehicle, closed, date_in, date_out, r_labor, r_tax, hazmat, labor, parts, tax, total) ") +
            QString("values ('" + owner + "', '" + vehicle + "', 'N', 'now', 'now', '" + 
                             Config.readEntry("/CroftJ/AutoHouse/LaborRate") + "', '" + 
                             Config.readEntry("/CroftJ/AutoHouse/SalesTaxRate") + "', '0.0', '0.0', '0.0', '0.0', '0.0')");

      printf("InvoiceWindow::createWorkOrder(): sql = |%s|\n", (const char*)sql);
      QSqlQuery wo1_q(sql);
      if ( !wo1_q.isActive() ) 
      {
         QMessageBox::warning(NULL, "Error!", "Error creating new Workorder: \n" + wo1_q.lastError().driverText() + wo1_q.lastError().databaseText(), "Okay");
         return;
      }
      sql = "select currval('workorders_ident_seq')";
      QSqlQuery wo2_q(sql);
      if ( !wo2_q.isActive() || !wo2_q.next()) 
      {
         QMessageBox::warning(NULL, "Error!", "Cannot find new Workorder: \n" + wo2_q.lastError().driverText() + wo2_q.lastError().databaseText(), "Okay");
         return;
      }
      refresh(wo2_q.value(0).toString());
   }
   else 
   {
      QMessageBox::warning(NULL, "Error!", "Cannot find owner for selected vehicle!\n" + vehc_q.lastError().driverText() + vehc_q.lastError().databaseText(), "Okay");
      return;
   }
}


void InvoiceWindow::editRates()
{
   Configuration *conf = new Configuration();
   conf->initialize();
   conf->exec();
   delete conf;
}

/**********************************************************************/
/*   Here  we  set  the main font for the application. It is assumed  */
/*   that the fonts of the children fonts will be based of of it      */
/**********************************************************************/
void InvoiceWindow::configureFont()
{
   bool t;

   printf("InvoiceWindow::configureFont(): Enter\n");
   QFont f = a->font();
   f = QFontDialog::getFont(&t, f);
   if ( t ) 
   {
      a->setFont(f, true);
      QString font = f.family();
      QString weight;
      weight.setNum(f.weight());
      font += "::" + weight;
      QString pointsize;
      pointsize.setNum(f.pointSize());
      font += "::" + pointsize;
      QString italic;
      italic.setNum(f.italic());
      font += "::" + italic;
      printf("InvoiceWindow::configureFont(): font string |%s|\n", (const char*)font);
      Config.writeEntry("/CroftJ/AutoHouse/Font", font);
   }
   printf("InvoiceWindow::configureFont(): Enter\n");
}


/**********************************************************************/
/*   This member function opens the WOPartsEntry Dialog box. After the  */
/*   dialogcloses, the Parts Window on the main form is repainted.    */
/**********************************************************************/
void InvoiceWindow::addWOParts()
{
   if ( WorkOrderEntry->isClosed() ) 
   {
      QString s = "<qt><p><nobr>This workorder has already been closed!</nobr><br><nobr><b>Updating it will affect your accounting.</b></nobr></p><p><nobr>Are you sure you want to update it?</nobr></p></qt>";
      if (QMessageBox::critical(NULL, "Updating Closed Work Order", s, QMessageBox::Yes, 
                                 QMessageBox::No | QMessageBox::Escape | QMessageBox::Default) == QMessageBox::No) 
      {
         printf("InvoiceWindow::updateRecord(): Exit\n");
         return;
      }
   }

   WOPartsEntry *pe = new WOPartsEntry(this);
   pe->woPartsForm1->setWorkOrder(WorkOrderEntry->getWorkorder());

   pe->woPartsTable1->setFilter("workorder='" + WorkOrderEntry->getWorkorder() + "'");
   pe->woPartsTable1->refresh();
   pe->woPartsTable1->selectRow(0);
   pe->exec();
   QString sql = "select sum(total) from wo_parts_v where workorder = '" + WorkOrderEntry->getWorkorder() + "'";
   QSqlQuery q(sql);
   if ( q.isActive() ) 
   {
      while (q.next()) 
      {
         sql = "update workorders set parts = '" + q.value(0).toString() + "' where ident = '" + WorkOrderEntry->getWorkorder() + "'"; 
         QSqlQuery q1(sql);
         q1.exec();
      }
   }
   refresh(WorkOrderEntry->getWorkorder());
   pe->woPartsTable1->setFocus();
}


/**********************************************************************/
/*   This member function opens the LaborEntry Dialog box. After the  */
/*   dialogcloses, the Labor Window on the main form is repainted.    */
/**********************************************************************/
void InvoiceWindow::addWOLabor()
{
   if ( WorkOrderEntry->isClosed() ) 
   {
      QString s = "<qt><p><nobr>This workorder has already been closed!</nobr><br><nobr><b>Updating it will affect your accounting.</b></nobr></p><p><nobr>Are you sure you want to update it?</nobr></p></qt>";
      if (QMessageBox::critical(NULL, "Updating Closed Work Order", s, QMessageBox::Yes, 
                                 QMessageBox::No | QMessageBox::Escape | QMessageBox::Default) == QMessageBox::No) 
      {
         printf("InvoiceWindow::updateRecord(): Exit\n");
         return;
      }
   }

   LaborEntry *pe = new LaborEntry(this);
   pe->woLaborForm->setWorkOrder(WorkOrderEntry->getWorkorder());

   pe->woLaborTable->setFilter("workorder='" + WorkOrderEntry->getWorkorder() + "'");
   pe->woLaborTable->refresh();
   pe->woLaborTable->selectRow(0);

   pe->exec();
   QString sql = "select sum(cost) from wo_labor_v where workorder = '" + WorkOrderEntry->getWorkorder() + "'";
   QSqlQuery q(sql);
   if ( q.isActive() ) 
   {
      while (q.next()) 
      {
         sql = "update workorders set labor = '" + q.value(0).toString() + "' where ident = '" + WorkOrderEntry->getWorkorder() + "'"; 
         QSqlQuery q1(sql);
         q1.exec();
      }
   }
   refresh(WorkOrderEntry->getWorkorder());
   pe->woLaborTable->setFocus();
}

void InvoiceWindow::cancelChanges()
{
   refresh(WorkOrderEntry->getWorkorder());
}


void InvoiceWindow::printWorkOrder()
{
   PreviewWorkOrder *pvw = new PreviewWorkOrder(this);
   pvw->setWorkorder(WorkOrderEntry->getWorkorder());
   pvw->exec();
   QString s = "Do you want to close this work worder now?";
   if (QMessageBox::critical(NULL, "Close Work Order", s, QMessageBox::Yes | QMessageBox::Default, 
                              QMessageBox::No | QMessageBox::Escape) == QMessageBox::Yes) 
   {
      closeWorkOrder();
   }
}


void InvoiceWindow::moveWorkOrders()
{
   MoveTransactions mt(this);
   mt.exec();
}


void InvoiceWindow::startAccounting()
{
   QString sAcctName = Config.readEntry("/CroftJ/AutoHouse/AccountingBin");
   QProcess p(sAcctName, this);
//   a->connect(p, SIGNAL(processExited()), this, SLOT(accountingExited()));
   p.start();
}

void InvoiceWindow::accountingExited()
{

}


void InvoiceWindow::doReport(int rpt)
{
   QString reportName = Reports[rpt];

   printf("Have request for report %d:|%s|\n", rpt, (const char*)reportName);
   fflush(stdout);
   ReportGen *rptGen = new ReportGen(this);
   rptGen->setReport(autohouse, &reportName);
   rptGen->exec();
}


void InvoiceWindow::dataDump()
{
//   DataDump *d = new DataDump;
//   d->exec();
}

void InvoiceWindow::dataRestore()
{
//   DataRestore *d = new DataRestore;
//   d->exec();
}


void InvoiceWindow::editVendors()
{
   VendorEntry *v = new VendorEntry();
   v->initialize();
   v->exec();
   delete v;
}


void InvoiceWindow::editParts()
{
   PartsEdit *p = new PartsEdit();
   p->initialize();
   p->exec();
   delete p;
}


void InvoiceWindow::closeWorkOrder()
{
   printf("InvoiceWindow::closeWorkOrder(): Enter\n");
   QString s = "Are you sure you want to close this work order?";
   if (QMessageBox::critical(NULL, "Close Work Order", s, QMessageBox::Yes | QMessageBox::Default, 
                              QMessageBox::No | QMessageBox::Escape) == QMessageBox::Yes) 
   {
      QString sql;
      printf("InvoiceWindow::closeWorkOrder(): Starting transaction\n");
      autohouse->transaction();

      sql = "select od_out, vehicle from workorders where ident = '" + WorkOrderEntry->getWorkorder() + "'";
      printf("InvoiceWindow::closeWorkOrder(): sql = |%s|\n", (const char*)sql);
      QSqlQuery q0(sql);
      if ( q0.isActive() ) 
      {
         if (q0.next()) 
         {
            QString od_out = q0.value(0).toString();
            QString vehicle = q0.value(1).toString();
            sql = "update vehicles set odom = '" + od_out + "' where ident = '" + vehicle + "'";
            printf("InvoiceWindow::closeWorkOrder(): sql = |%s|\n", (const char*)sql);
            QSqlQuery qx(sql);
         }
      }

      sql = "update workorders set closed = 'Y' where ident = '" + WorkOrderEntry->getWorkorder() + "'";
      printf("InvoiceWindow::closeWorkOrder(): sql = |%s|\n", (const char*)sql);
      QSqlQuery q(sql);
      sql = "select pn, qty from wo_parts where workorder = '" + WorkOrderEntry->getWorkorder() + "'";
      printf("InvoiceWindow::closeWorkOrder(): sql = |%s|\n", (const char*)sql);
      QSqlQuery q1(sql);
      if ( q1.isActive() ) 
      {
         while (q1.next()) 
         {
            QString pn = q1.value(0).toString();
            QString qty = q1.value(1).toString();
            if ( pn != "" ) 
            {
               printf("InvoiceWindow::closeWorkOrder(): Removing %s items from count for pn %s\n", (const char*)qty, (const char*)pn);
               sql = "update parts set onhand = onhand - '" + qty + "' where pn = '" + pn + "'";
               printf("InvoiceWindow::closeWorkOrder(): sql = |%s|\n", (const char*)sql);
               QSqlQuery q2(sql);
            }
         }
      }
      printf("InvoiceWindow::closeWorkOrder(): Commiting transaction\n");
      autohouse->commit();
      clearForm();
   }
   printf("InvoiceWindow::closeWorkOrder(): Exit\n");
}


void InvoiceWindow::deleteWorkOrder()
{
   printf("InvoiceWindow::deleteWorkOrder(): Enter\n");
   printf("InvoiceWindow::deleteWorkOrder(): Workorder Ident: %d\n", (const char*)WorkOrderEntry->getWorkorder());
   if (WorkOrderEntry->getWorkorder().toLong() > 0)
   {
      printf("InvoiceWindow::deleteWorkOrder(): Deleting workorder\n");
      WorkOrderEntry->deleteRecord();
   }
   clearForm();
   printf("InvoiceWindow::deleteWorkOrder(): Exit\n");
}


void InvoiceWindow::clearForm()
{
   printf("InvoiceWindow::clearForm(): Enter\n");
   Customer->clearForm();
   Vehicle->clearForm();
   WorkOrderEntry->clearForm();
   PartsWindow->clear();
   LaborWindow->clear();
   PartsTotal->setText(MoneyEdit::formatCurrency(0.00));
   LaborTotal->setText(MoneyEdit::formatCurrency(0.00));
   printf("InvoiceWindow::clearForm(): Exit\n");
}


void InvoiceWindow::keyPressEvent( QKeyEvent * e )
{
   printf("InvoiceWindow::keyPressEvent(): Entry, have key press for %s\n", e->text().isNull() == true ? "NULL" : (const char*)e->text());
   if ( e->key() == Qt::Key_Escape ) 
   {
      e->accept();
//      done(QDialog::Rejected);
   }
   printf("InvoiceWindow::keyPressEvent(): Exit\n");
}


void InvoiceWindow::editBizInfo()
{
   BizInfo *conf = new BizInfo;
   conf->initialize();
   conf->exec();
   delete conf;
}
